"""Convert a pyproject.toml file to a requirements.txt file."""

from __future__ import annotations
from typing import Any

import toml


def _convert_setuptools_to_requirements(
    parsed_toml_file: dict[str, Any],
    *,
    optional_lists: list[str] | None,
) -> str:
    project: dict | None = parsed_toml_file.get("project")

    if project is None:
        raise RuntimeError(
            "The project section is missing from the TOML file. Exiting..."
        )

    dependencies = set(project.get("dependencies", []))

    if optional_lists is not None:
        optional_dependency_list: dict[str, Any] = project.get(
            "optional-dependencies", {}
        )
        optional_lists_to_include: set[str] = (
            set(optional_lists) if optional_lists is not None else set()
        )

        for optional_list, deps in optional_dependency_list.items():
            if optional_list in optional_lists_to_include:
                dependencies.update(deps)

    return "\n".join(sorted(dependencies))


def convert_toml_to_requirements(
    toml_file_string: str,
    *,
    optional_lists: list[str] | None,
    poetry: bool,
) -> str:
    """Convert a pyproject.toml file to a requirements.txt file.

    Args:
        toml_file_string: The contents of the pyproject.toml file.
        optional_lists: A list of optional dependency lists to include.
        poetry: Whether to use poetry instead of setuptools.

    Returns:
        The contents of the requirements.txt file.

    Raises:
        RuntimeError: If poetry is set to True since it is not yet supported.
    """
    if poetry:
        raise RuntimeError("Poetry is not yet supported.")

    parsed_toml_file: dict[str, Any] = toml.loads(toml_file_string)

    return _convert_setuptools_to_requirements(
        parsed_toml_file,
        optional_lists=optional_lists,
    )



from __future__ import annotations
import argparse
from pathlib import Path
from typing import NamedTuple


class CLIArguments(NamedTuple):
    toml_file_path: Path
    requirements_file_path: Path
    optional_lists: list[str] | None
    poetry: bool = False


def get_parsed_arguments() -> CLIArguments:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--toml-file",
        "-t",
        type=str,
        default="pyproject.toml",
        help="The path of the pyproject.toml file to convert.",
    )
    parser.add_argument(
        "--requirements-file",
        "-r",
        type=str,
        default="requirements.txt",
        help="The path of the requirements file to generate.",
    )
    parser.add_argument(
        "--poetry",
        "-p",
        help="If the TOML file is a poetry project.",
        action="store_true",
    )
    parser.add_argument(
        "-o",
        "--optional-lists",
        help="Comma separated optional dependency lists (or poetry extras / groups) to include.",
    )
    args: argparse.Namespace = parser.parse_args()

    return CLIArguments(
        toml_file_path=Path(args.toml_file),
        requirements_file_path=Path(args.requirements_file),
        optional_lists=args.optional_lists.split(",") if args.optional_lists else None,
        poetry=args.poetry or False,
    )



"""Main entry point for the CLI."""

from __future__ import annotations

from toml_to_requirements.cli import CLIArguments, get_parsed_arguments
from toml_to_requirements.convert_toml_to_requirements import (
    convert_toml_to_requirements,
)


def main() -> None:
    arguments: CLIArguments = get_parsed_arguments()

    if not arguments.toml_file_path.is_file():
        raise FileNotFoundError(f"File not found: {arguments.toml_file_path}")

    with arguments.toml_file_path.open("r") as f:
        toml_file_contents: str = f.read()

    requirements_content: str = convert_toml_to_requirements(
        toml_file_contents,
        optional_lists=arguments.optional_lists,
        poetry=arguments.poetry,
    )

    with arguments.requirements_file_path.open("w") as f:
        f.write(requirements_content)
